require 'spec_helper'

Test 'Subledger' do

  let(:factory   ) { Subledger::Factory.create }
  let(:amount    ) { '1' }

  balances_order = { :before   => [ Subledger::Domain::Debit.new( '3' ),
                                    Subledger::Domain::Debit.new( '1' ) ],
                     :ending   => [ Subledger::Domain::Debit.new( '6' ),
                                    Subledger::Domain::Debit.new( '3' ) ],
                     :starting => [ Subledger::Domain::Debit.new( '6' ),
                                    Subledger::Domain::Debit.new( '10' ) ],
                     :after    => [ Subledger::Domain::Debit.new( '10' ),
                                    Subledger::Domain::Debit.new( '15' ) ] }

  balances_id = { :preceding => [ Subledger::Domain::Debit.new( '3' ),
                                  Subledger::Domain::Debit.new( '1' ) ],
                  :following => [ Subledger::Domain::Debit.new( '10' ),
                                  Subledger::Domain::Debit.new( '15' ) ] }

  Instance do
    subject { factory.client }

    Test 'Account' do

      # POST /books/:book_id/accounts
      RespondsTo 'account.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveAccount' do
          subject.account.create( factory.account_args ).
                     must_be_instance_of Subledger::Domain::ActiveAccount
        end
      end

      # GET /books/:book_id/accounts/:id/first_and_last_line
      [ :archived, :active ].each do |state|
        RespondsTo 'account.first_and_last_line' do
          let(:account) { factory.create_account state }

          When 'the account has no lines' do
            ByReturning "an empty array" do
              subject.
                account.
                  first_and_last_line( :id => account.id ).must_be_instance_of Array

              subject.
                account.
                  first_and_last_line( :id => account.id ).empty?.must_equal true
            end
          end

          When 'the account has lines' do
            before { factory.create_posted_line_collection account }

            ByReturning "an empty array" do
              subject.
                account.
                  first_and_last_line( :id => account.id ).must_be_instance_of Array

              subject.
                account.
                  first_and_last_line( :id => account.id ).
                    length.must_equal 2

              subject.
                account.
                  first_and_last_line( :id => account.id ).
                    first.must_be_instance_of Subledger::Domain::PostedLine

              subject.
                account.
                  first_and_last_line( :id => account.id ).
                    last.must_be_instance_of Subledger::Domain::PostedLine
            end
          end
        end
      end

      [ :archived, :active ].each do |state|
        RespondsTo 'account.read when description has a / in it' do
          let(:description) { 'this has a / in it' }
          let(:args       ) { factory.account_args.
                                        merge :description => description }
          let(:account    ) { subject.account.create args }

          ByReturning "an instance of #{state} account" do
            subject.account.read( :id => account.id ).id.must_equal account.id
            subject.account.read( :id => account.id ).description.must_equal description
          end
        end
      end

      # PATCH /books/:book_id/accounts/:id
      [ :archived, :active ].each do |state|
        RespondsTo 'accounts.update' do
          let(:account    ) { factory.create_account state }
          let(:args       ) { account.attributes }
          let(:description) { 'updated description' }
          let(:update_args) { args.merge :description => description,
                                         :version     => 2 }

          ByReturning "an instance of #{state} account" do
            patched = subject.accounts.update update_args

            patched.must_be_instance_of account.class
            patched.description.must_equal description
          end
        end
      end

      # GET /books/:book_id/accounts ? before   = :description & limit = :limit & state = :state
      #                                ending
      #                                starting
      #                                after
      [ :active, :archived ].each do |state|
        Subledger.scenarios_order.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "accounts.collect :action => #{action}, :state => #{state}" do
            let(:accounts) { factory.create_account_collection state }
            let(:item    ) { accounts[2] }
            let(:limit   ) { 2 }
            let(:klass   ) { item.class }

            let(:args) { { :action      => action,
                           :description => item.description,
                           :state       => state,
                           :limit       => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} accounts" do
                subject.accounts.collect( args ).length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} accounts" do
                count = 0

                subject.accounts.collect args do |account|
                  expectation = expectations[count]

                  account.description.must_equal expectation

                  account.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /accounts ? preceding = :book_id & limit = :limit & state = :state
      #                 following = :book_id & limit = :limit & state = :state
      [ :active, :archived ].each do |state|
        Subledger.scenarios_id.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "accounts.collect :action => #{action}" do
            let(:accounts) { factory.create_account_collection state }
            let(:item    ) { accounts[2] }
            let(:limit   ) { 2 }
            let(:klass   ) { item.class }

            let(:args) { { :action => action,
                           :id     => item.id,
                           :state  => state,
                           :limit  => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} accounts" do
                subject.accounts.collect( args ).length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} accounts" do
                count = 0

                subject.accounts.collect args do |account|
                  expectation = expectations[count]

                  account.description.must_equal expectation

                  account.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /books/:book_id/accounts/:id/lines ? before   = :effective_at & limit = :limit
      #                                          ending
      #                                          starting
      #                                          after
      Subledger.scenarios_order.each do |scenario|
        action       = scenario.keys.first
        expectations = scenario[action]
        balances     = balances_order[action]

        RespondsTo "account.read(:id => id).lines action => #{action}" do
          let(:account) { factory.create_account :active }
          let(:lines  ) { factory.create_posted_line_collection account }
          let(:item   ) { lines[2] }
          let(:limit  ) { 2 }

          let(:args) { { :action       => action,
                         :effective_at => item.effective_at,
                         :limit        => limit } }

          When 'not given a block' do
            ByReturning 'a list of account lines' do
              account.lines( args ).length.must_equal expectations.length
            end
          end

          When 'given a block' do
            ByYielding 'a list of account lines' do
              count = 0

              account.lines args do |line|
                expectation = expectations[count]
                line.description.must_equal expectation

                line.must_be_instance_of Subledger::Domain::PostedLine

                line.posted_at.wont_be_nil
                line.effective_at.wont_be_nil

                line.balance.must_be_instance_of Subledger::Domain::Balance

                line.balance.must_equal balances[count]

                count += 1
              end

              count.must_equal expectations.length
            end
          end
        end
      end

      # GET /books/:book_id/accounts/:id/lines ? preceding = :line_id & limit = :limit
      #                                          following = :line_id & limit = :limit
      Subledger.scenarios_id.each do |scenario|
        action       = scenario.keys.first
        expectations = scenario[action]
        balances     = balances_id[action]

        RespondsTo "account_instance.lines action => #{action}" do
          let(:account) { factory.create_account :active }
          let(:lines  ) { factory.create_posted_line_collection account }
          let(:item   ) { lines[2] }
          let(:limit  ) { 2 }

          let(:args) { { :action => action,
                         :id     => item.id,
                         :limit  => limit } }

          When 'not given a block' do
            ByReturning 'a list of account lines' do
              account.lines( args ).length.must_equal expectations.length
            end
          end

          When 'given a block' do
            ByYielding 'a list of account lines' do
              count = 0

              account.lines args do |line|
                expectation = expectations[count]

                line.description.must_equal expectation

                line.must_be_instance_of Subledger::Domain::PostedLine

                line.posted_at.wont_be_nil
                line.effective_at.wont_be_nil

                line.balance.must_be_instance_of Subledger::Domain::Balance

                line.balance.must_equal balances[count]

                count += 1
              end

              count.must_equal expectations.length
            end
          end
        end
      end

      # GET /books/:book_id/accounts/:account_id/lines/:id

      RespondsTo 'account.line.read' do
        let(:account) { factory.create_account :active }
        let(:lines  ) { factory.create_posted_line_collection account }
        let(:line   ) { lines[2] }

        ByReturning 'Subledger::Domain::PostedLine' do
          posted_line = account.line( :id => line.id ).read
          posted_line.must_be_instance_of Subledger::Domain::PostedLine
          posted_line.posted_at.must_be_instance_of Time
          posted_line.effective_at.must_be_instance_of Time

          if Subledger::STORE == :api
            posted_line.balance.must_be_instance_of Subledger::Domain::Balance
            posted_line.balance.must_equal factory.
                                             client.
                                               debit( '6' )
          end
        end
      end

      # POST /books/:book_id/accounts/:id/archive
      RespondsTo 'account_instance.archive' do
        ByReturning 'an instance of Subledger::Domain::ArchivedAccount' do
          factory.create_account(:active).archive.
                     must_be_instance_of Subledger::Domain::ArchivedAccount
        end
      end

      # POST /books/:book_id/archived_accounts/:id/activate
      RespondsTo 'archived_account_instance.activate' do
        ByReturning 'an instance of Subledger::Domain::ActiveAccount' do
          factory.create_account(:archived).activate.
                     must_be_instance_of Subledger::Domain::ActiveAccount
        end
      end

      # GET /books/:book_id/accounts/:id/balance/:at
      [ :archived, :active ].each do |state|
        RespondsTo 'account_instance.balance' do
          let(:account) { factory.create_account state }

          ByReturning 'a Subledger::Domain::Balance' do
            account.balance( :at => Time.now ).must_be_instance_of Subledger::Domain::Balance
          end
        end
      end
    end
  end
end
