require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }
  let(:amount ) { '1' }

  Instance do
    subject { factory.client }

    Test 'Category' do

      # POST /books/:book_id/categories
      RespondsTo 'category.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveCategory' do
          subject.category.create( factory.category_args ).
                     must_be_instance_of Subledger::Domain::ActiveCategory
        end
      end

      # GET /books/:book_id/categories/:id
      [ :archived, :active ].each do |state|
        RespondsTo 'category.read' do
          let(:category) { factory.create_category state }

          ByReturning "an instance of #{state} category" do
            subject.category.read( :id => category.id ).
                       must_be_instance_of category.class
          end
        end
      end

      # PATCH /books/:book_id/categories/:id
      [ :archived, :active ].each do |state|
        RespondsTo 'categories.update' do
          let(:category   ) { factory.create_category state }
          let(:args       ) { category.attributes }
          let(:description) { 'updated description' }
          let(:update_args) { args.merge :description => description,
                                         :version     => 2 }

          ByReturning "an instance of #{state} category" do
            patched = subject.categories.update update_args

            patched.must_be_instance_of category.class
            patched.description.must_equal description
          end
        end
      end

      # GET /books/:book_id/categories ? before   = :description & limit = :limit & state = :state
      #                                  ending
      #                                  starting
      #                                  after
      [ :active, :archived ].each do |state|
        Subledger.scenarios_order.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "categories.collect :action => #{action}, :state => #{state}" do
            let(:categories) { factory.create_category_collection state }
            let(:item      ) { categories[2] }
            let(:limit     ) { 2 }
            let(:collection) { subject.categories.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action      => action,
                           :description => item.description,
                           :state       => state,
                           :limit       => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} categories" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} categories" do
                count = 0

                collection.each do |category|
                  expectation = expectations[count]

                  category.description.must_equal expectation

                  category.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /categories ? preceding = :book_id & limit = :limit & state = :state
      #                   following = :book_id & limit = :limit & state = :state
      [ :active, :archived ].each do |state|
        Subledger.scenarios_id.each do |scenario|
          action       = scenario.keys.first
          expectations = scenario[action]

          RespondsTo "categories.collect :action => #{action}" do
            let(:categories) { factory.create_category_collection state }
            let(:item      ) { categories[2] }
            let(:limit     ) { 2 }
            let(:collection) { subject.categories.collect args }
            let(:klass     ) { item.class }

            let(:args) { { :action => action,
                           :id     => item.id,
                           :state  => state,
                           :limit  => limit } }

            When 'not given a block' do
              ByReturning "a list of #{state} categories" do
                collection.length.must_equal limit
              end
            end

            When 'given a block' do
              ByYielding "a list of #{state} categories" do
                count = 0

                collection.each do |category|
                  expectation = expectations[count]

                  category.description.must_equal expectation

                  category.must_be_instance_of klass

                  count += 1
                end

                count.must_equal limit
              end
            end
          end
        end
      end

      # GET /books/:book_id/categories/:id/accounts
      if Subledger::STORE == :api
        RespondsTo 'category_instance.accounts' do
          ByRaising 'a Store::ReportError' do
            lambda do
              factory.create_category(:active).accounts
            end.must_raise Subledger::Store::CategoryError
          end
        end
      else
        RespondsTo "category_instance.accounts" do
          let(:category) { factory.create_category :active }

          let(:category_accounts) {  }

          When 'not given a block' do
            ByReturning 'a list of accounts' #do
            #   category.accounts.length.must_equal category_accounts.length
            # end
          end

          When 'given a block' do
            ByYielding 'a list of accounts' #do
            #   count = 0
            #
            #   category.accounts do |account|
            #     expectation = category_accounts[count]
            #
            #     account.description.must_equal expectation.description
            #
            #     account.must_be_instance_of Subledger::Domain::ActiveAccount
            #
            #     count += 1
            #   end
            #
            #   count.must_equal category_accounts.length
            # end
          end
        end
      end

      # POST /books/:book_id/categories/:id/archive
      RespondsTo 'category_instance.archive' do
        ByReturning 'an instance of Subledger::Domain::ArchivedCategory' do
          factory.create_category(:active).archive.
                     must_be_instance_of Subledger::Domain::ArchivedCategory
        end
      end

      # POST /books/:book_id/archived_categories/:id/activate
      RespondsTo 'archived_category_instance.activate' do
        ByReturning 'an instance of Subledger::Domain::ActiveCategory' do
          factory.create_category(:archived).activate.
                     must_be_instance_of Subledger::Domain::ActiveCategory
        end
      end
    end
  end
end
