require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }

  Instance do
    subject { factory.client }

    Test 'Identities' do
      # POST /identities
      RespondsTo 'identities.create' do
        ByReturning 'an instance of ActiveIdentity and ActiveKey' do
          identity, key = subject.identities.create factory.identity_args
          identity.must_be_instance_of Subledger::Domain::ActiveIdentity
          key.must_be_instance_of Subledger::Domain::ActiveKey
        end
      end

      # GET /identities/:id
      RespondsTo 'identities.read' do
        let(:identity) { factory.identity }

        ByReturning 'a kind of Subledger::Domain::ActiveIdentity' do
          subject.identities.read( :id => identity.id ).
                          must_be_instance_of Subledger::Domain::ActiveIdentity
        end
      end

      # PATCH /identities/:id
      RespondsTo 'identities.update' do
        let(:identity   ) { factory.identity }
        let(:description) { 'updated description' }
        let(:args       ) { identity.attributes }
        let(:update_args) { args.merge :description => description,
                                       :version     => 2 }

        ByReturning 'an updated kind of Subledger::Domain::ActiveIdentity' do
          patched = subject.identities.update update_args

          patched.must_be_instance_of Subledger::Domain::ActiveIdentity
          patched.description.must_equal description
        end
      end
    end
  end
end
