require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }
  let(:identity) { factory.identity }

  Instance do
    subject { factory.client }

    Test 'Key' do
      # POST /identities/:identity_id/keys
      RespondsTo 'keys.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveKey' do
          subject.keys.create( factory.key_args( identity ) ).
                         must_be_instance_of Subledger::Domain::ActiveKey
        end
      end

      # POST /identities/:identity_id/keys/:key_id/archive
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_key_instance.archive" do
          let(:key) { factory.create_key state, identity }

          ByReturning 'an instance of Subledger::Domain::ArchivedKey' do
            key.archive.must_be_instance_of Subledger::Domain::ArchivedKey
          end
        end
      end

      # POST /identities/:identity_id/keys/:key_id/activate
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_key_instance.activate" do
          let(:key) { factory.create_key state, identity }

          ByReturning 'an instance of Subledger::Domain::ActiveKey' do
            key.activate.must_be_instance_of Subledger::Domain::ActiveKey
          end
        end
      end
    end
  end
end
