require 'spec_helper'

Test 'Subledger' do

  let(:factory) { Subledger::Factory.create }

  Instance do
    subject { factory.client }

    Test 'Org' do
      # POST /orgs
      RespondsTo 'orgs.create' do
        ByReturning 'an instance of Subledger::Domain::ActiveOrg' do
          subject.orgs.create( factory.org_args ).
                         must_be_instance_of Subledger::Domain::ActiveOrg
        end
      end

      # GET /orgs/:id
      RespondsTo 'orgs.read' do
        let(:org) { factory.create_org :active }

        ByReturning 'a kind of Subledger::Domain::ActiveOrg' do
          subject.orgs.read( :id => org.id ).
                          must_be_instance_of Subledger::Domain::ActiveOrg
        end
      end

      # PATCH /orgs/:id
      RespondsTo 'orgs.update' do
        let(:org       ) { factory.create_org :active }
        let(:description) { 'updated description' }
        let(:args       ) { org.attributes }
        let(:update_args) { args.merge :description => description,
                                       :version     => 2 }

        ByReturning 'an updated kind of Subledger::Domain::ActiveOrg' do
          patched = subject.orgs.update update_args

          patched.must_be_instance_of Subledger::Domain::ActiveOrg
          patched.description.must_equal description
        end
      end

      # POST /orgs/:id/archive
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_org_instance.archive" do
          let(:org) { factory.create_org state }

          ByReturning 'an instance of Subledger::Domain::ArchivedOrg' do
            org.archive.must_be_instance_of Subledger::Domain::ArchivedOrg
          end
        end
      end

      # POST /orgs/:id/activate
      [ :archived, :active ].each do |state|
        RespondsTo "#{state}_org_instance.activate" do
          let(:org) { factory.create_org state }

          ByReturning 'an instance of Subledger::Domain::ActiveOrg' do
            org.activate.must_be_instance_of Subledger::Domain::ActiveOrg
          end
        end
      end
    end
  end
end
