require 'spec_helper'

Test 'Subledger' do

  let(:amount ) { '1' }

  let(:args) { { } }

  Module do
    subject { Subledger }

    RespondsTo 'new' do
      When ':store is not received' do
        ByReturning 'an instance of Subledger::Interface::Client' do
          subject.new( args ).must_be_instance_of Subledger::Interface::Client
        end
      end

      When ':store is received' do
        And ':store is a symbol' do
          ByReturning 'an instance of Subledger::Interface::Client' do
            args.merge! :store => Subledger::STORE

            subject.new( args ).must_be_instance_of Subledger::Interface::Client
          end
        end

        And ':store is unknown' do
          ByRaising 'a Error' do
            args.merge! :store => 'unknown'

            lambda { subject.new( args ) }.must_raise Subledger::Error
          end
        end
      end
    end

    RespondsTo 'parse_time' do
      ByReturning 'a UTC Time' do
        [
          ( Subledger::EARLIEST_TIME ).iso8601(3),
          ( Subledger::LATEST_TIME   ).iso8601(3),
        ].each do |time_string|
          subject.parse_time(
                    :test,
                    time_string ).utc?.must_equal true
        end

        [
          nil,
          '',
          'bob',
          ( Subledger::EARLIEST_TIME - 1 ).iso8601(3),
          ( Subledger::LATEST_TIME + 1   ).iso8601(3),
        ].each do |time_string|
          lambda { subject.parse_time(
                             :test,
                             time_string ) }.must_raise Subledger::DateError
        end
      end
    end
  end

  Instance do
    subject { Subledger.new args }

    RespondsTo 'debit' do
      When 'an amount is received' do
        ByReturning 'a kind of Debit' do
          subject.debit( amount ).must_be_kind_of Subledger::Domain::Debit
        end
      end

      When 'an amount is not received' do
        ByReturning 'an ancestor of Debit' do
          subject.debit.ancestors.must_include Subledger::Domain::Debit
        end
      end
    end

    RespondsTo 'credit' do
      When 'an amount is received' do
        ByReturning 'a kind of Debit' do
          subject.credit( amount ).must_be_kind_of Subledger::Domain::Credit
        end
      end

      When 'an amount is not received' do
        ByReturning 'an ancestor of Debit' do
          subject.credit.ancestors.must_include Subledger::Domain::Credit
        end
      end
    end

    RespondsTo 'zero' do
      ByReturning 'a kind of Zero' do
        subject.zero( 0 ).must_be_instance_of Subledger::Domain::Zero
      end
    end
  end
end
